/*

   This used to be

   chorus effect with simple feedback mechanism
   lags output by 1 sample, even in dry mode (!!!)

   the dry/wet slider allows negative values, those
   are applied to the wet signal only (to get a phase-
   cancellation-at-minimum-delay kind of chorus). the
   dry signal is always passed on without sign changed

   the lfo minimum modulation depth gets substracted
   from the overall maximum modulation depth, so the
   maximum depth is the same whatever min. depth you
   specify.

   by TraumFlug

   but I changed it quite a bit. 

   - ld 9.4.2010

 */
#include "simpledelay.h"
#include <lunar/fx.hpp>
#include <lunar/dsp.h>

//maximum depth in samples, for buffer sizes
//
#define MAX_DEPTH 262100
//
//make sure this is at least larger than the depths
//accessible in the manifest.xml


class simpledelay : public lunar::fx<simpledelay>
{
public: //hey, free, open source software is always "public" ;)

	//variables

	//history sample buffers dry & wet
	unsigned int buf_size;  //size of the 2 buffers
	unsigned int buf_wrap;  //mask for wraparound
	float *dbufl; //dry input buffers
	float *dbufr;
	unsigned int rpos;     //buffer(s) read position
	unsigned int wpos;     //buffer(s) write position

	//allocate buffers sized by power of two with enough space for desired depth
	void alloc_buffers(int maxdepth) {
		//find minimum size for buffer to hold full delay line
		int minsize = maxdepth + 4; //+4 just to be on the safe side... ;)
		int bits=8;
		while( (minsize - (1<<bits)) > 0 ) bits++;
		buf_size = 1 << bits;
		buf_wrap = buf_size - 1;
		//allocate & set zero
		dbufl = new float[buf_size];
		dbufr = new float[buf_size];
		for(int i = 0; i < buf_size; i++)
		{
			dbufl[i] = 0.0;
			dbufr[i] = 0.0;    
		}
	}

	void init() {
		//buffers
		dbufl = dbufr = 0;
		alloc_buffers(MAX_DEPTH);
		rpos = 0;
		wpos = 0;
	}

	void exit() {
		if(dbufl) delete[] dbufl;
		if(dbufr) delete[] dbufr;	
	}

	void process_events() {
		if(globals->delay) {
			rpos = (wpos - (*globals->delay)) & buf_wrap;
		}
	}


	void process_stereo(float *inL, float *inR, float *outL, float *outR, int n) {
		for(int i = 0; i < n; i++) {
			outL[i] = dbufl[rpos];
			outR[i] = dbufr[rpos];
			dbufl[wpos] = inL[i];
			dbufr[wpos] = inR[i];
			rpos = (rpos + 1) & buf_wrap;
			wpos = (wpos + 1) & buf_wrap;
		}


	}
};

lunar_fx *new_fx() { return new simpledelay(); }

